<?php

namespace Shpopping\Product\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Shpopping\Product\Forms\Product as ProductForm;
use Shpopping\Product\Http\Requests\ProductResourceRequest;
use Shpopping\Product\Http\Resources\ProductResource;
use Shpopping\Product\Http\Resources\ProductsCollection;
use Shpopping\Product\Models\Product;
use Shpopping\Product\Actions\ProductAction;
use Shpopping\Product\Actions\ProductActions;


/**
 * Resource controller class for product.
 */
class ProductResourceController extends BaseController
{

    /**
     * Initialize product resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = ProductForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('shpopping.product.modules'), 'product', guard_url('product'));
            return $next($request);
        });
    }

    /**
     * Display a list of product.
     *
     * @return Response
     */
    public function index(ProductResourceRequest $request)
    {
        $request = $request->all();
        $page = ProductActions::run('paginate', $request);

        $data = new ProductsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('product::product.names'))
            ->view('product::product.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display product.
     *
     * @param Request $request
     * @param Model   $product
     *
     * @return Response
     */
    public function show(ProductResourceRequest $request, Product $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ProductResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('product::product.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('product::product.show')
            ->output();
    }

    /**
     * Show the form for creating a new product.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ProductResourceRequest $request, Product $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ProductResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('product::product.name'))
            ->view('product::product.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new product.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ProductResourceRequest $request, Product $model)
    {
        try {
            $request = $request->all();
            $model = ProductAction::run('store', $model, $request);
            $data = new ProductResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('product::product.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('product/product/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/product/product'))
                ->redirect();
        }

    }

    /**
     * Show product for editing.
     *
     * @param Request $request
     * @param Model   $product
     *
     * @return Response
     */
    public function edit(ProductResourceRequest $request, Product $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ProductResource($model);
        // return view('product::product.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('product::product.name'))
            ->view('product::product.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the product.
     *
     * @param Request $request
     * @param Model   $product
     *
     * @return Response
     */
    public function update(ProductResourceRequest $request, Product $model)
    {
        try {
            $request = $request->all();
            $model = ProductAction::run('update', $model, $request);
            $data = new ProductResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('product::product.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('product/product/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('product/product/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the product.
     *
     * @param Model   $product
     *
     * @return Response
     */
    public function destroy(ProductResourceRequest $request, Product $model)
    {
        try {

            $request = $request->all();
            $model = ProductAction::run('destroy', $model, $request);
            $data = new ProductResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('product::product.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('product/product/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('product/product/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
